// $Id$

//=============================================================================
/**
 *  @file    dump_memory.cpp
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include "jyugem/bee.h"
#include "jyugem/bee/dump_memory.h"
using namespace jyugem::bee;

namespace jyugem { namespace bee
{

BCHAR * dump_words::line_init =
  _TX("1234 1234  1234 1234  1234 1234  1234 1234  : 1234 1234 1234 1234");

BCHAR * dump_bytes::line_init =
  _TX("12 12 12 12 12 12 12 12  12 12 12 12 12 12 12 12 : 12345678 12345678");

#ifndef BOUNDARY
#define BOUNDARY     0
#define NO_BOUNDARY  1
#endif

BCHAR * lower_code = _TX("0123456789abcdef");
BCHAR * upper_code = _TX("0123456789ABCDEF");

/*---------------------------------------------------------------------------*/
/* dump memory                                                               */
/*---------------------------------------------------------------------------*/
template<typename T>
void dump_memory<T>::operator()(BCHAR * startp, size_t size,
                                std::string& hexstr)
{
  BCHAR * endp = startp + size;
  BCHAR * curptr;
  UINT    dumpnt;
  BCHAR   c;
  BCHAR * hexp;
  BCHAR * charp;
  BCHAR linebuf[128];
  long base = 0;                    // offset position

  hexstr = _TX("");
  curptr = startp;

  if (m_line != null)
    _tcscpy(linebuf, m_line);
  else
    memset(m_line, 0, sizeof(linebuf));

  //------------------------------------------------------------------
  // dump hex data
  //------------------------------------------------------------------
  for ( ; curptr < endp; curptr += 16)
  {
    hexp = (BCHAR *)(linebuf + m_hexp);
    charp = (BCHAR *)(linebuf + m_charp);

    for (dumpnt = 0; *(hexp + dumpnt) != '\0'; dumpnt++)
      *(hexp + dumpnt) = ' ';
    *(hexp + m_sepap) = ':';

    for (dumpnt = 0; dumpnt < 16; dumpnt++)
    {
      c = *(curptr + dumpnt);

      this->nextp(dumpnt, hexp, charp);

      //----------------------------------------------------------
      // dump character
      //----------------------------------------------------------
      if ((curptr + dumpnt) < startp || (curptr + dumpnt) >= endp)
      {
        hexp += 2;
        charp++;
      }
      else
      {
        *hexp++ = m_hexcodep[(c >> 4) & 0x0F];
        *hexp++ = m_hexcodep[c & 0x0F];
        if (c < ' ' || c > '~')
          *charp++ = '.';
        else
          *charp++ = c;
      }
    }

    //--------------------------------------------------------------
    // formatting
    //--------------------------------------------------------------
    if (! (m_mode & DUMP_CHAR))
    { // Cut printed character code
      linebuf[m_sepap] = '\0';
    }
    if (m_mode & DUMP_OFFSET)
    {
      BCHAR buf[128];
      _stprintf(buf, _TX("%08lx "), base);
      hexstr += buf;
    }

    hexstr += linebuf;                        // copy hex data

    if ((m_mode & DUMP_CHAR) || (m_mode & DUMP_NEWLINE))
    {
      if (m_mode & DUMP_CONT)
      {
        hexstr += '\\';
      }
      hexstr += '\n';
    }

    base += (16 / m_size);
  }
}

} }  // *namespace* - jyugem::bee