// $Id$

//=============================================================================
/**
 *  @file    fileio.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include "jyugem/bee/fileio.h"
#include "ace/FILE_Addr.h"
#include "ace/FILE_Connector.h"

namespace jyugem { namespace bee
{

//-----------------------------------------------------------------------------
// Open the file
//-----------------------------------------------------------------------------
int fileio::open(void * argv)
{
  TRACE_FUNCTION();
  
  if (m_path.size() <= 0)
  {
    TRACE_DEBUG((_TX("Empty file name\n")));
    return -1;
  }
  ACE_FILE_Connector connector;
  int result = connector.connect(m_file,
                         ACE_FILE_Addr(m_path.c_str()),
                         0,                 // No timeout.
                         ACE_Addr::sap_any, // Ignored.
                         0,                 // Don't try to reuse the addr.
                         O_RDWR | O_CREAT,
                         ACE_DEFAULT_FILE_PERMS);
  if (result < 0)
  {
    TRACE_ERROR((_TX("Error connect text file (%s)\n"),
                 m_path.c_str()));
    return -1;
  }

  // TRACE_DEBUG((_TX("Connect file : %s\n"), m_path.c_str()));

  m_opened = true;
  return 0;
}

//-----------------------------------------------------------------------------
// Write data
//-----------------------------------------------------------------------------
int fileio::write(const std::string& textstr)
{
  TRACE_FUNCTION();
  
  if (m_file.send_n(textstr.c_str(), textstr.size()) == -1)
  {
    TRACE_ERROR((_TX("%p\n"), _TX("file.send_n()")));
    return -1;
  }
  return 0;
}

//-----------------------------------------------------------------------------
int fileio::write(const BYTE * buf, size_t size)
{
  TRACE_FUNCTION();
  if (m_file.send_n(buf, size) == -1)
  {
    TRACE_ERROR((_TX("%p\n"), _TX("file.send_n()")));
    return -1;
  }
  return 0;
}

//-----------------------------------------------------------------------------
// Read
//-----------------------------------------------------------------------------
int fileio::read(BYTE * buf, size_t size)
{
  TRACE_FUNCTION();
  if (m_file.recv(buf, size) == -1)
  {
    TRACE_ERROR((_TX("%p\n"), _TX("file.read()")));
    return -1;
  }
  return 0;
}

//-----------------------------------------------------------------------------
// Get size
//-----------------------------------------------------------------------------
size_t fileio::size() const
{
  TRACE_FUNCTION();
  ACE_FILE_Info finfo;
  if (const_cast<ACE_FILE_IO&>(m_file).get_info(finfo) == -1)
  {
    TRACE_ERROR((_TX("%p\n"), _TX("file.get_info()")));
    return 0;
  }
  return (size_t)finfo.size_;
}

} } // *namespace** jyugem::bee
