// $Id$

//=============================================================================
/**
 *  @file  meta_primitive.cpp
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include <boost/lexical_cast.hpp>
#include "jyugem/bee.h"
#include "jyugem/bee/value.h"
#include "meta_template.h"
using namespace jyugem::bee;

namespace jyugem { namespace bee
{

template<typename T>
bool metadata_t<T>::equal(const bee::value& lhs,
  const bee::value& rhs)
{
  TRACE_FUNCTION();
  try
  {
    T * lptr = const_cast<bee::value&>(lhs).any_cast<T>();
    T * rptr = const_cast<bee::value&>(rhs).any_cast<T>();
    return (*lptr == *rptr);
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return false;
}

template<typename T>
bool metadata_t<T>::less(const bee::value& lhs,
  const bee::value& rhs)
{
  TRACE_FUNCTION();
  try
  {
    T * lptr = const_cast<bee::value&>(lhs).any_cast<T>();
    T * rptr = const_cast<bee::value&>(rhs).any_cast<T>();
    return (*lptr < *rptr);
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return false;
}

template<typename T>
bool metadata_t<T>::getBool(const bee::value& v) const
{
  TRACE_FUNCTION();
  try
  {
    T * vptr = const_cast<bee::value&>(v).any_cast<T>();
    return (*vptr != 0) ? true : false;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return false;
}

bool metadata_t<std::string>::getBool(const bee::value& v) const
{
  TRACE_FUNCTION();
  try
  {
    T * vptr = const_cast<bee::value&>(v).any_cast<T>();
    if (*vptr == _TX("TRUE") || *vptr == _TX("true") ||
      *vptr == _TX("True"))
    {
      return true;
    }
    return false;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return false;
}


template<>
bool metadata_t<bool>::getBool(const bee::value& v) const
{
  TRACE_FUNCTION();
  bool result = false;
  try
  {
    result = const_cast<bee::value&>(v).value_cast<bool>();
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return result;
}

template<typename T>
int metadata_t<T>::getInt(const bee::value& v) const
{
  TRACE_FUNCTION();
  try
  {
    T * vptr = const_cast<bee::value&>(v).any_cast<T>();
    return (int)(*vptr);
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return -1;
}

int metadata_t<std::string>::getInt(const bee::value& v) const
{
  TRACE_FUNCTION();
  try
  {
    std::string * vptr = const_cast<bee::value&>(v).any_cast<std::string>();
    int result = boost::lexical_cast<int>(vptr);
    return result;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  catch(const boost::bad_lexical_cast& lex)
  {
    TRACE_DEBUG((_TX("%s\n"), lex.what()));
  }
  return -1;
}

template<typename T>
UINT metadata_t<T>::getUInt(const value& v) const
{
  TRACE_FUNCTION();
  try
  {
    T * vptr = const_cast<bee::value&>(v).any_cast<T>();
    return (UINT)(*vptr);
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  return 0;
}

UINT metadata_t<std::string>::getUInt(const value& v) const
{
  TRACE_FUNCTION();
  try
  {
    std::string * vptr = const_cast<bee::value&>(v).any_cast<std::string>();
    UINT result = boost::lexical_cast<UINT>(vptr);
    return result;
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
  }
  catch(const boost::bad_lexical_cast& lex)
  {
    TRACE_DEBUG((_TX("%s\n"), lex.what()));
  }
  return 0;
}

template<typename T>
int metadata_t<T>::setInt(bee::value& v, int val)
{
  v = (T)val;
  return 0;
}

int metadata_t<bool>::setInt(bee::value& v, int val)
{
  v = (val != 0) ? true : false;
  return 0;
}

int metadata_t<std::string>::setInt(bee::value& v, int val)
{
  TRACE_FUNCTION();
  try
  {
    std::string * vptr = const_cast<bee::value&>(v).any_cast<std::string>();
    *vptr = boost::lexical_cast<std::string>(val);
  }
  catch (const boost::bad_lexical_cast & lex)
  {
    TRACE_DEBUG((_TX("%s\n"), lex.what()));
  }
  return 0;
}


template<typename T>
int metadata_t<T>::set(bee::value& v, const value& from)
{
  TRACE_FUNCTION();
  if (v.type() == from.type())
  {
    try
    {
      T * lptr = v.any_cast<T>();
      T * rptr = const_cast<bee::value&>(from).any_cast<T>();
      *lptr = *rptr;
    }
    catch (const boost::bad_any_cast& )
    {
      TRACE_ERROR((_TX("bad_any_cast \n")));
      return -1;
    }
  }
  else
  {
    TRACE_DEBUG((_TX("Type is different(%d & %d)\n"),
          v.type(), from.type()));
    v.setInt(from.getInt());
  }
  return 0;
}

int metadata_t<bool>::set(bee::value& v, const value& from)
{
  v = from.getBool();
  return 0;
}

int metadata_t<std::string>::set(bee::value& v, const value& from)
{
  TRACE_FUNCTION();
  if (v.type() == from.type())
  {
    try
    {
      std::string * lptr = v.any_cast<std::string>();
      std::string * rptr = const_cast<bee::value&>(from).any_cast<std::string>();
      *lptr = *rptr;
    }
    catch (const boost::bad_any_cast& )
    {
      TRACE_ERROR((_TX("bad_any_cast \n")));
      return -1;
    }
  }
  else
  {
    TRACE_DEBUG((_TX("Type is different(%d & %d)\n"),
          v.type(), from.type()));
    std::string valstr;
    from.get(valstr);
    v = valstr;
  }
  return 0;
}

template<typename T>
int metadata_t<T>::set(bee::value& v, const string& valstr)
{
  TRACE_FUNCTION();
  try
  {
    T newval = boost::lexical_cast<T>(valstr);
    v = newval;
  }
  catch(const boost::bad_lexical_cast& lex)
  {
    TRACE_DEBUG((_TX("%s\n"), lex.what()));
    return -1;
  }
  return 0;
}

template<typename T>
int metadata_t<T>::set(bee::value& v, const BYTE * bin, size_t q, int order)
{
  TRACE_FUNCTION();
  if (q == 0 || bin == null || sizeof(T) > q)
    return -1;

  if (order == ENDIAN || sizeof(T) == 1)
  {
    T * ptr = (T *)bin;
    v = *ptr;
    return sizeof(T);
  }
  else if (sizeof(T) == 2)
  {
    T newval  = (*bin << 8) + (*(bin + 1));
    v = newval;
  }
  else if (sizeof(T) == 4)
  {
    T newval;
    BYTE * bufptr = (BYTE *)&newval;
    *(bufptr + 3) = *(bin);
    *(bufptr + 2) = *(bin + 1);
    *(bufptr + 1) = *(bin + 2);
    *(bufptr)   = *(bin + 3);
    v = newval;
  }
  else if (sizeof(T) == 8)
  {
    T newval;
    BYTE * bufptr = (BYTE *)&newval;
    *(bufptr + 7) = *(bin);
    *(bufptr + 6) = *(bin + 1);
    *(bufptr + 5) = *(bin + 2);
    *(bufptr + 4) = *(bin + 3);
    *(bufptr + 3) = *(bin + 4);
    *(bufptr + 2) = *(bin + 5);
    *(bufptr + 1) = *(bin + 6);
    *(bufptr)   = *(bin + 7);
    v = newval;
  }
  else
  {
    TRACE_DEBUG((_TX("Illegal value size(%d)\n"), sizeof(T)));
    return -1;
  }
  return sizeof(T);
}

int metadata_t<std::string>::set(bee::value& v,
  const BYTE * bin, size_t q, int order)
{
  TRACE_FUNCTION();
  if (q == 0 || bin == null)
    return -1;

  std::string * vptr;
  try
  {
    vptr = v.any_cast<std::string>();
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_ERROR((_TX("bad_any_cast \n")));
    return -1;
  }

  if (order == ENDIAN || sizeof(T) == 1)
  {
    std::string buf((BCHAR *)bin);
    vptr->assign(buf);  
  }
  else 
  { // Ignore endian
    std::string buf((BCHAR *)bin);
    vptr->assign(buf);  
  }
  return vptr->size();
}

int metadata_t<bool>::set(bee::value& v, const BYTE * bin, size_t q, int order)
{
  TRACE_FUNCTION();
  if (q == 0 || bin == null)
    return -1;

  v = ((*bin) != 0) ? true : false;
  return 0;
}

template<typename T>
int metadata_t<T>::get(const bee::value& v, BYTE * bin, size_t q, int order)
{
  TRACE_FUNCTION();
  if (q == 0 || bin == null || sizeof(T) > q)
    return -1;

  if (order == ENDIAN || sizeof(T) == 1)
  {
    *bin = (BYTE)v.getInt();
    return 0;
  }

  try
  {
    BYTE * vptr = (BYTE *)(const_cast<bee::value&>(v).any_cast<T>());
    if (sizeof(T) == 2)
    {
      *bin = *(vptr + 1);
      *(bin + 1) = *vptr;
    }
    else if (sizeof(T) == 4)
    {
      *(bin + 3) = *(vptr);
      *(bin + 2) = *(vptr + 1);
      *(bin + 1) = *(vptr + 2);
      *(bin)   = *(vptr + 3);
    }
    else if (sizeof(T) == 8)
    {
      *(bin + 7) = *(vptr);
      *(bin + 6) = *(vptr + 1);
      *(bin + 5) = *(vptr + 2);
      *(bin + 4) = *(vptr + 3);
      *(bin + 3) = *(vptr + 4);
      *(bin + 2) = *(vptr + 5);
      *(bin + 1) = *(vptr + 6);
      *(bin)   = *(vptr + 7);
    }
    else
    {
      TRACE_DEBUG((_TX("Illegal value size(%d)\n"), sizeof(T)));
      return -1;
    }
  }
  catch (const boost::bad_any_cast& )
  {
    TRACE_DEBUG((_TX("bad_any_cast\n")));
    return -1;
  }
  return 0;
}

template<typename T>
int metadata_t<T>::get(const bee::value& v, std::string& retstr)
{
  TRACE_FUNCTION();
  try
  {
    T val = const_cast<bee::value&>(v).value_cast<T>();
    retstr = boost::lexical_cast<std::string>(val);
  }
  catch (const boost::bad_lexical_cast & lex)
  {
    TRACE_DEBUG((_TX("%s\n"), lex.what()));
  }
  return 0;
}

template<typename T>
void metadata_t<T>::copy(bee::value& v, const bee::value& rhs)
{
  TRACE_FUNCTION();
  if (v.type() == rhs.type())
  {
    try
    {
      T * lptr = v.any_cast<T>();
      T * rptr = const_cast<bee::value&>(rhs).any_cast<T>();
      *lptr = *rptr;
    }
    catch (const boost::bad_any_cast& )
    {
      TRACE_ERROR((_TX("bad_any_cast \n")));
      return ;
    }
  }
  else
  {
    TRACE_DEBUG((_TX("Type is different(%d & %d)\n"),
          v.type(), rhs.type()));
    v.setInt(rhs.getInt());
  }
}

void metadata_t<bool>::copy(bee::value& v, const bee::value& rhs)
{
  v = rhs.getBool();
}

// bytes in list
template<typename T>
size_t metadata_t<T>::length(const bee::value& ) const
{
  return m_bytes;
}


template<typename T>
int metadata_t<T>::make(value& outv)
{
  T initval = 0;
  outv = initval;
  return 0;
}

int metadata_t<bool>::make(value& outv)
{
  outv = false;
  return 0;
}

int metadata_t<std::string>::make(value& outv)
{
  std::string initval(_TX(""));
  outv = initval;
  return 0;
}

} }  // *namespace* - jyugem::bee

