// $Id$

//=============================================================================
/**
 *  @file    Carrier.cpp
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#pragma warning( disable: 4244 ) // '__w64 int'
#endif


#include "state/CarrierState.h"
#include "jyugem/cms/Carrier.h"
#include "jyugem/gem/event_manager.h"

namespace bee = jyugem::bee;
namespace gem = jyugem::gem;

//
namespace jyugem { namespace cms
{

//
// Carrier
//
Carrier::Carrier() : bee::state_task("Carrier"),
  m_BypassReadID(false), m_Capacity(25), m_SubstrateCount(0),
  m_CarrierIDStatus(ID_NOT_READ), m_CarrierAccessingStatus(NOT_ACCESSED),
  m_SlotMapStatus(SLOTMAP_NOT_READ), m_LocationID(_TX("")), m_Usage(_TX(""))
{
  m_status = new CarrierState(this);

  for (size_t i = 0; i < m_Capacity; i++)
  {
    m_ContentMap.push_back(null);
    m_SlotMap.push_back(UNDEFINED);
  }
}

Carrier::~Carrier()
{
  CarrierState * cs = static_cast<CarrierState *>(m_status);
  cs->terminate();
  delete cs;
}

#if 0
void Carrier::copy(const Carrier& rhs)
{
  this->bee::state_task::operator=(rhs);
  //
  // transit current_state();
  //
  m_BypassReadID = rhs.m_BypassReadID;
  m_Capacity = rhs.m_Capacity;
  m_SubstrateCount = rhs.m_SubstrateCount;
  m_CarrierIDStatus = rhs.m_CarrierIDStatus;
  m_CarrierAccessingStatus = rhs.m_CarrierAccessingStatus;
  m_SlotMapStatus = rhs.m_SlotMapStatus;
  m_LocationID = rhs.m_LocationID;
  m_Usage = rhs.m_Usage;
}
#endif

extern gem::event_manager evtmngr;

int Carrier::open0()
{
  int result = this->open(null);
  this->on_trigger_connect(boost::bind(&gem::event_manager::notify, &evtmngr, _1, _2));

  CarrierState * cs = static_cast<CarrierState *>(m_status);
  cs->initiate();

  return 0;
}

const BCHAR * Carrier::current_state(Orthogonal region)
{
  CarrierState * cs = static_cast<CarrierState *>(m_status);
  if (region > Orthogonal::ACCESS)
  {
    TRACE_FAIL((_TX("Illegl region number.")));
    return null;
  }
  const cstr_id * stp = cs->get_current_name(int(region));
  if (stp)
    return stp->name();
  return null;
}


} }  // *namespace* jyugem::cms

