// $Id$

//=============================================================================
/**
 *  @file    LoadPort.cpp
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#pragma warning( disable: 4244 ) // '__w64 int'
#endif


#include "jyugem/cms/LoadPort.h"
#include "jyugem/cms/Carrier.h"
#include "state/LoadPortState.h"
#include "jyugem/gem/event_manager.h"

namespace bee = jyugem::bee;
namespace gem = jyugem::gem;

//
namespace jyugem { namespace cms
{

//
// LoadPort
//
LoadPort::LoadPort(bee::byte portid) : bee::state_task(""),
  m_bypassReadID(false),
  m_portTransferState(0), m_accessMode(BOTH_ACCESS), m_manualTimer(5000),
  m_manualInType(SWITCH_SWITCH), m_manualOutType(SWITCH_SWITCH),
  m_undockStyle(CARRIER_COMPLETE), m_unclampStyle(CARRIER_COMPLETE),
  m_carrier(), m_reservationState(0), m_portAssosiationState(0),
  m_portID(portid), m_reason(0)
{
  m_status = new LoadPortState(this);
  m_uniqueName = FORMATER1("LoadPort-%1%", int(portid));
}

LoadPort::~LoadPort()
{
  LoadPortState * lps = static_cast<LoadPortState *>(m_status);
  lps->terminate();
  delete lps;
}

#if 0
void LoadPort::copy(const LoadPort& rhs)
{
  this->bee::state_task::operator=(rhs);
  //
  // transit current_state();
  //
  m_bypassReadID = rhs.m_bypassReadID;
  m_portTransferState = rhs.m_portTransferState;
  m_accessMode = rhs.m_accessMode;
  m_manualTimer = rhs.m_manualTimer;
  m_manualInType = rhs.m_manualInType;
  m_manualOutType = rhs.m_manualOutType;
  m_undockStyle = rhs.m_undockStyle;
  m_unclampStyle = rhs.m_unclampStyle;
  m_carrier = rhs.m_carrier;
  m_reservationState = rhs.m_reservationState;
  m_portAssosiationState = rhsm_portAssosiationState;
  m_portID = rhs.m_portID;
  m_reason = rhs.m_reason;
}
#endif

extern gem::event_manager evtmngr;

int LoadPort::open0()
{
  int result = this->open(null);
  this->on_trigger_connect(boost::bind(&gem::event_manager::notify, &evtmngr, _1, _2));

  LoadPortState * lps = static_cast<LoadPortState *>(m_status);
  lps->initiate();

  return result;
}

const BCHAR * LoadPort::current_state(Orthogonal region)
{
  LoadPortState * lps = static_cast<LoadPortState *>(m_status);
  if (region > Orthogonal::ASSOCIATION)
  {
    TRACE_FAIL((_TX("Illegl region number.")));
    return null;
  }
  const cstr_id * stp = lps->get_current_name(int(region));
  if (stp)
    return stp->name();
  return null;
}

int LoadPort::onCarrier()
{
  Carrier * carrier = new Carrier();
  this->m_carrier.reset(carrier);
  this->m_carrier->open0();
  return 0;
}

int LoadPort::offCarrier()
{
  //
  // Become OutOfService status
  //

  this->m_carrier.reset();
  return 0;
}

} }  // *namespace* jyugem::cms

