// $Id$

//////////////////////////////////////////////////////////////////////////////
// Copyright (c) Andreas Huber Doenni 2002-2004.
// Use, modification and distribution are subject to the Boost Software
// License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
//////////////////////////////////////////////////////////////////////////////

//=============================================================================
/**
 *  @file    LoadPortState.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#pragma warning( disable: 4244 ) // '__w64 int'
#endif

#include "LoadPortState.h"
#include "jyugem/cms/LoadPort.h"
#include "jyugem/cms/Carrier.h"

namespace bee = jyugem::bee;
namespace fsm = boost::fsm;
namespace mpl = boost::mpl;

//
//
namespace jyugem { namespace cms
{

//-----------------------------------------------------------------------------
// State Machine
//
LoadPortState::LoadPortState(LoadPort * ldp)
    : task_base(ldp->uniqueName(), ldp)
{
}

LoadPortState::task_base::event_dictionary  LoadPortState::_event_dictionary;

static cstr_id _initLoadPortStatus_( "InitLoadPortStatus", 1 );
static cstr_id _loadPortTransfer_  ( "LoadPortTransfer", 2 );
static cstr_id _accessMode_        ( "AccessMode", 3 );
static cstr_id _reservation_       ( "Reservation", 4 );
static cstr_id _carrierAssociation_( "CarrierAssociation", 5 );
static cstr_id _outOfService_      ( "OutOfService", 6 );
static cstr_id _inService_         ( "InService", 7 );
static cstr_id _transferReady_     ( "TransferReady", 8 );
static cstr_id _readyToLoad_       ( "ReadyToLoad", 9 );
static cstr_id _readyToUnload_     ( "ReadyToUnload", 10 );
static cstr_id _transferBlocked_   ( "TransferBlocked", 11 );
static cstr_id _manual_            ( "Manual", 12 );
static cstr_id _auto_              ( "Auto", 13 );
static cstr_id _notReserved_       ( "NotReserved", 14 );
static cstr_id _reserved_          ( "Reserved", 15 );
static cstr_id _notAssociated_     ( "NotAssociated", 16 );
static cstr_id _associated_        ( "Associated", 17 );
static cstr_id _transferCondition_ ( "TransferCondition", 18 );

void LoadPortState::InitEventDictionary()
{
  if (! _event_dictionary.empty())
    return ;

  _event_dictionary.insert(std::make_pair("auto", new EvChangeAccess(EvChangeAccess::AUTO)));
  _event_dictionary.insert(std::make_pair("manual", new EvChangeAccess(EvChangeAccess::MANUAL)));
  _event_dictionary.insert(std::make_pair("bind",   new EvBind()));
  _event_dictionary.insert(std::make_pair("in-service", new EvChangeServiceStatus(EvChangeServiceStatus::IN_SERVICE)));
  _event_dictionary.insert(std::make_pair("out-of-service", new EvChangeServiceStatus(EvChangeServiceStatus::OUT_OF_SERVICE)));
  _event_dictionary.insert(std::make_pair("ready",  new EvReady()));
  _event_dictionary.insert(std::make_pair("notify", new EvCarrierNotification()));
  _event_dictionary.insert(std::make_pair("process",new EvProcComplete()));
  _event_dictionary.insert(std::make_pair("unload", new EvUnloadComplete()));
  _event_dictionary.insert(std::make_pair("fail",   new EvTransferFail()));
  _event_dictionary.insert(std::make_pair("recreate",    new EvCarrierReCreate()));
  _event_dictionary.insert(std::make_pair("release",     new EvCarrierRelease()));
  _event_dictionary.insert(std::make_pair("tag-write",   new EvCarrierTagWriteData()));
  _event_dictionary.insert(std::make_pair("carrier-in",  new EvCarrierIn()));
  _event_dictionary.insert(std::make_pair("carrier-out", new EvCarrierOut()));
  _event_dictionary.insert(std::make_pair("cancel-all",    new EvCancelAllCarrierOut()));
  _event_dictionary.insert(std::make_pair("cancel-bind",   new EvCancelBind()));
  _event_dictionary.insert(std::make_pair("cancel-carrier",new EvCancelCarrier()));
  _event_dictionary.insert(std::make_pair("cancel-port",   new EvCancelCarrierAtPort()));
  _event_dictionary.insert(std::make_pair("cancel-notify", new EvCancelCarrierNotification()));
  _event_dictionary.insert(std::make_pair("cancel-out",    new EvCancelCarrierOut()));
  _event_dictionary.insert(std::make_pair("cancel-reserve",new EvCancelReservationAtPort()));

  // Initialize static name of states
  InitLoadPortStatus::custom_static_type_ptr( &_initLoadPortStatus_ );
  LoadPortTransfer::custom_static_type_ptr( &_loadPortTransfer_ );
  AccessMode::   custom_static_type_ptr( &_accessMode_ );
  Reservation::  custom_static_type_ptr( &_reservation_ );
  CarrierAssociation::custom_static_type_ptr( &_carrierAssociation_ );
  OutOfService:: custom_static_type_ptr( &_outOfService_ );
  InService::    custom_static_type_ptr( &_inService_ );
  TransferReady::custom_static_type_ptr( &_transferReady_ );
  TransferCondition::custom_static_type_ptr( &_transferCondition_ );
  ReadyToLoad::  custom_static_type_ptr( &_readyToLoad_ );
  ReadyToUnload::custom_static_type_ptr( &_readyToUnload_ );
  TransferBlocked::custom_static_type_ptr( &_transferBlocked_ );
  Manual::       custom_static_type_ptr( &_manual_ );
  Auto::         custom_static_type_ptr( &_auto_ );
  NotReserved::  custom_static_type_ptr( &_notReserved_ );
  Reserved::     custom_static_type_ptr( &_reserved_ );
  NotAssociated::custom_static_type_ptr( &_notAssociated_ );
  Associated::   custom_static_type_ptr( &_associated_ );

}

//
const cstr_id * LoadPortState::get_current_name(int orthogonal)
{
  const cstr_id * result = null;
  if (orthogonal > LoadPort::Orthogonal::ASSOCIATION)
  {
    return result;
  }
  switch (orthogonal)
  {
  case LoadPort::Orthogonal::TRANSFER :
    result = this->current_state_name<ViewedState>();
    break;
  case LoadPort::Orthogonal::ACCESS :
    result = this->current_state_name<AccessViewed>();
    break;
  case LoadPort::Orthogonal::RESERVATION :
    result = this->current_state_name<ReservationViewed>();
    break;
  case LoadPort::Orthogonal::ASSOCIATION :
    result = this->current_state_name<AssociationViewed>();
    break;
  }
  return result;
}

//
//
fsm::result TransferCondition::react(const EvEvalCondition&)
{
  LoadPort * lp = outermost_context().cast_task<LoadPort>();
  Carrier * carrier = lp->m_carrier.get();
  if (carrier == null)
  {
    return transit< ReadyToLoad >();
  }
  else if (carrier != null &&
           (carrier->m_CarrierAccessingStatus == Carrier::CARRIER_COMPLETE ||
            carrier->m_CarrierAccessingStatus == Carrier::CARRIER_STOPPED))
  {
    return transit< ReadyToUnload >();
  }
  else
    return transit< TransferBlocked >();
}

} }  // *namespace* jyugem::cms

