// $Id$

//////////////////////////////////////////////////////////////////////////////
// Copyright (c) Andreas Huber Doenni 2002-2004.
// Use, modification and distribution are subject to the Boost Software
// License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
//////////////////////////////////////////////////////////////////////////////

//=============================================================================
/**
 *  @file    ProcessJobState.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif

#include "ProcessJobState.h"
#include "jyugem/pjm/ProcessJob.h"

namespace bee = jyugem::bee;
namespace fsm = boost::fsm;
namespace mpl = boost::mpl;

//
namespace jyugem { namespace pjm
{

//
//-----------------------------------------------------------------------------
// State Machine
//
ProcessJobState::ProcessJobState(ProcessJob * pjm)
  : task_base(pjm->uniqueName(), pjm), m_PRProcessStart(pjm->m_PRProcessStart)
{
}

ProcessJobState::task_base::event_dictionary  ProcessJobState::_event_dictionary;

static cstr_id _queued_and_pooled_( "QueuedAndPooled", 1 );
static cstr_id _active_( "Active", 2 );
static cstr_id _departed_( "Departed", 3 );
static cstr_id _settingup_( "SettingUp", 4 );
static cstr_id _executing_( "Executing", 5 );
static cstr_id _stopping_( "Stopping", 6 );
static cstr_id _aborting_( "Aborting", 7 );
static cstr_id _waitingForStart_( "WaitingForStart", 8 );
static cstr_id _processing_( "Processing", 9 );
static cstr_id _process_completed_( "ProcessCompleted", 10 );
static cstr_id _pause_( "Pause", 11 );
static cstr_id _pausing_( "Pausing", 12 );
static cstr_id _paused_( "Paused", 13 );


void ProcessJobState::InitEventDictionary()
{
  if (! _event_dictionary.empty())
    return ;

  _event_dictionary.insert(std::make_pair("abort",   new EvAbort()));
  _event_dictionary.insert(std::make_pair("aborted", new EvAborted()));
  _event_dictionary.insert(std::make_pair("alloc",   new EvAllocate()));
  _event_dictionary.insert(std::make_pair("cancel",  new EvCancel()));
  _event_dictionary.insert(std::make_pair("depart",  new EvDepart()));
  _event_dictionary.insert(std::make_pair("complete",new EvCompleted()));
  _event_dictionary.insert(std::make_pair("pause",   new EvPause()));
  _event_dictionary.insert(std::make_pair("paused",  new EvPaused()));
  _event_dictionary.insert(std::make_pair("resume",  new EvResume()));
  _event_dictionary.insert(std::make_pair("start",   new EvStart()));
  _event_dictionary.insert(std::make_pair("stop",    new EvStop()));
  _event_dictionary.insert(std::make_pair("stopped", new EvStopped()));

  // Initialize static name of states
  Departed::custom_static_type_ptr( &_departed_ );
  QueuedAndPooled::custom_static_type_ptr( &_queued_and_pooled_ );
  WaitingForStart::custom_static_type_ptr( &_waitingForStart_ );
  Processing::custom_static_type_ptr     ( &_processing_ );
  ProcessCompleted::custom_static_type_ptr( &_process_completed_ );
  Active::custom_static_type_ptr   ( &_active_ );
  Executing::custom_static_type_ptr( &_executing_ );
  SettingUp::custom_static_type_ptr( &_settingup_ );
  Stopping::custom_static_type_ptr ( &_stopping_ );
  Aborting::custom_static_type_ptr ( &_aborting_ );
  Pause::custom_static_type_ptr    ( &_pause_ );
  Pausing::custom_static_type_ptr  ( &_pausing_ );
  Paused::custom_static_type_ptr   ( &_paused_ );
}


} }  // *namespace* jyugem::pjm
